package gq.unurled.raxen.config;

import com.mongodb.client.MongoCollection;
import com.mongodb.client.model.Filters;
import gq.unurled.raxen.Raxen;
import gq.unurled.raxen.components.player.Attributes.Attribute;
import gq.unurled.raxen.components.player.Attributes.Attributes;
import gq.unurled.raxen.components.player.Inventories;
import gq.unurled.raxen.components.player.Storage;
import gq.unurled.raxen.components.player.Storages.EnderChest;
import gq.unurled.raxen.manager.ProfileManager;
import gq.unurled.raxen.manager.StorageManager;
import gq.unurled.raxen.utils.Items;
import gq.unurled.raxen.utils.MongoDB;
import net.kyori.adventure.text.Component;
import org.bson.Document;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.InventoryType;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static gq.unurled.raxen.utils.Utils.log;
import static gq.unurled.raxen.utils.Utils.warn;

public class PlayerConfig {

    private static Raxen main;
    private MongoDB mongoDB;
    private MongoCollection<Document> mongoCollection;
    private static ProfileManager profileManager;
    private static StorageManager sto;

    public PlayerConfig(Raxen main) {
        PlayerConfig.main = main;
        sto = Raxen.getStorageManager();
        this.mongoDB = StorageManager.getMongo();
        this.mongoCollection = mongoDB.getMongoCollection();
        profileManager = Raxen.getProfileManager();
    }

    public void close()  {
        mongoDB.close();
    }

    public void saveUsingMongoDB(Player player, Attributes attributes, String invstr, Integer reverse) {
        Document doc = new Document("uuid", player.getUniqueId().toString())
                .append("name", player.getName())
                .append("health", attributes.getHealth(player))
                .append("itemHealth", attributes.getItemHealth(player))
                .append("defense", attributes.getDefense(player))
                .append("itemDefense", attributes.getItemDefense(player))
                .append("speed", attributes.getSpeed(player))
                .append("itemSpeed", attributes.getItemSpeed(player))
                .append("strength", attributes.getStrength(player))
                .append("itemDmg", attributes.getItemStrength(player))
                .append("inv", invstr);
        if (profileManager.getPlayerStorage(player.getUniqueId()) != null) {
            if(profileManager.getPlayerStorage(player.getUniqueId()).getEc() != null) {
                EnderChest ec = profileManager.getPlayerStorage(player.getUniqueId()).getEc();
                List<String> list = new ArrayList<String>();
                reverse = 0;
                for(ItemStack it : ec.getEc()) {
                    reverse += 1;
                    if(it != null) {
                        String s = Items.itemStackSerilize(it, reverse);
                        list.add(s);
                    }
                }
                String ecstr = Items.listItemStackSerelize(list);
                doc.append("ec", ecstr);
            }
        }
        Document playerDoc = mongoCollection.find(Filters.eq("uuid", player.getUniqueId().toString())).first();
        if(playerDoc == null) {
            mongoCollection.insertOne(doc);
            log("insert one");
        }
        else {
            mongoCollection.replaceOne(Filters.eq("uuid", player.getUniqueId().toString()), doc);
            log("replace one");
        }
    }

    public void saveUsingYml(Player player, Attributes attributes, String invstr, Integer reverse) {
        FileConfiguration config = sto.createYml(player);
        config.set("name", player.getName());
        config.set("health", attributes.getHealth(player));
        config.set("itemHealth", attributes.getItemHealth(player));
        config.set("strength", attributes.getStrength(player));
        config.set("itemDmg", attributes.getItemStrength(player));
        config.set("speed", attributes.getSpeed(player));
        config.set("itemSpeed", attributes.getItemSpeed(player));
        config.set("defense", attributes.getDefense(player));
        config.set("itemDefense", attributes.getItemDefense(player));
        config.set("inv", invstr);
        if (profileManager.getPlayerStorage(player.getUniqueId()) != null) {
            if(profileManager.getPlayerStorage(player.getUniqueId()).getEc() != null) {
                EnderChest ec = profileManager.getPlayerStorage(player.getUniqueId()).getEc();
                List<String> list = new ArrayList<String>();
                reverse = 0;
                for(ItemStack it : ec.getEc()) {
                    reverse += 1;
                    if(it != null) {
                        String s = Items.itemStackSerilize(it, reverse);
                        list.add(s);
                    }
                }
                String ecstr = Items.listItemStackSerelize(list);
                config.set("ec", ecstr);
            }
        }
        try {
            config.save(main.getDataFolder() + "/playerInfo/" + player.getUniqueId() + "/" + "playerInfo.yml");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void loadUsingMongoDB(Player player) {
        Document playerDoc = mongoCollection.find(Filters.eq("uuid", player.getUniqueId().toString())).first();
        if(playerDoc == null) {
            log("something went wrong");
            profileManager.createNewProfile(player);
            profileManager.createNewStorage(player);
            profileManager.createNewInventory(player);
            return;
        }
        Attributes.setAttributes(player, playerDoc.getInteger("health"), playerDoc.getInteger("itemHealth"), playerDoc.getInteger("defense"),
                playerDoc.getInteger("itemDefense"), playerDoc.getInteger("speed"), playerDoc.getInteger("itemSpeed"),
                playerDoc.getInteger("strength"), playerDoc.getInteger("itemDmg"));

        Inventory ec = Bukkit.createInventory(null, 54, Component.text("Ender Chest"));
        if(playerDoc.getString("ec") != null) {
            ItemStack[] itList = Items.listItemStackDeserilize(playerDoc.getString("ec"));
            ec.addItem(itList);
        }
        EnderChest ecc = new EnderChest(ec);
        Storage storage = new Storage(ecc);
        profileManager.setPlayerStorage(player.getUniqueId(), storage);
        Inventory inv = Bukkit.createInventory(player, InventoryType.PLAYER);
        ItemStack[] itListInv = Items.listItemStackDeserilize(playerDoc.getString("inv"));
        inv.addItem(itListInv);
        player.updateInventory();
        gq.unurled.raxen.components.player.Storages.Inventory invv = new gq.unurled.raxen.components.player.Storages.Inventory(inv);
        Inventories invvv = new Inventories(invv);
        profileManager.setPlayerInventory(player.getUniqueId(), invvv);
    }

    public void loadUsingYml(Player player) throws IOException {
        FileConfiguration config = sto.createYml(player);
        Attributes.setAttributes(player, config.getInt("health"), config.getInt("itemHealth"),
                config.getInt("defense"), config.getInt("itemDefense"),
                config.getInt("speed"), config.getInt("itemSpeed"),
                config.getInt("strength"), config.getInt("itemDmg"));

        Inventory ec = Bukkit.createInventory(null, 54, Component.text("Ender Chest"));
        if((String) config.get("ec") != null) {
            ItemStack[] itList = Items.listItemStackDeserilize((String) config.get("ec"));
            ec.addItem(itList);
        }
        EnderChest ecc = new EnderChest(ec);
        Storage storage = new Storage(ecc);
        profileManager.setPlayerStorage(player.getUniqueId(), storage);
        Inventory inv = Bukkit.createInventory(player, InventoryType.PLAYER);
        ItemStack[] itListInv = Items.listItemStackDeserilize((String) config.getString("inv"));
        inv.addItem(itListInv);
        player.updateInventory();
        gq.unurled.raxen.components.player.Storages.Inventory invv = new gq.unurled.raxen.components.player.Storages.Inventory(inv);
        Inventories invvv = new Inventories(invv);
        profileManager.setPlayerInventory(player.getUniqueId(), invvv);
    }

    public void savePlayerConfig(Player player) {
        Attributes attributes = new Attributes(main);
        Inventory inv = player.getInventory();
        List<String> listInv = new ArrayList<String>();
        Integer reverse = 0;
        for(ItemStack it : inv) {
            reverse = reverse + 1;
            if(it != null && it.getType() != Material.AIR) {
                String s = Items.itemStackSerilize(it, reverse);
                listInv.add(s);
            }
        }
        String invstr = Items.listItemStackSerelize(listInv);
        log("saving...");
        String storage = sto.getConfig().getString("storage");
        saveUsingMongoDB(player, attributes, invstr, reverse);
        log("using mongo");
        if (storage == "MYSQL" || storage == "YML") {
            saveUsingYml(player, attributes, invstr, reverse);
            log("using yml");
        }

        log("Player: " + player.getName() + " data successfully saved!");
    }

    public void loadPlayerConfig(Player player) {
        String st = StorageManager.getConfig().getString("storage");
        String value = "";
        if (st.equals("MONGODB")) {
            loadUsingMongoDB(player);
            value = "mongodb";
            warn("'" + st + "'");
        }
        else {
            warn("'" + st + "'");
            value = "yml";
            try {
                loadUsingYml(player);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        log("Player: " + player.getName() + " data successfully loaded using " + value);
    }

}
