package gq.unurled.raxen.utils;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import de.tr7zw.changeme.nbtapi.NBTItem;
import gq.unurled.raxen.components.items.Attributes;
import net.kyori.adventure.text.Component;
import org.bukkit.Material;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.entity.LivingEntity;
import org.bukkit.inventory.EntityEquipment;
import org.bukkit.inventory.ItemFlag;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import static gq.unurled.raxen.utils.Utils.color;
import static gq.unurled.raxen.utils.Utils.error;

public class Items {

    public static ItemStack createItem(Material material, int amount, boolean glow, boolean unb, String name, String... lore) {
        ItemStack item = new ItemStack(material, amount);
        ItemMeta itemMeta = item.getItemMeta();
        if(name != null) {
            itemMeta.displayName(Component.text(color(name)));
        }
        if(lore != null) {
            List<Component> list = new ArrayList<>();
            for(String string :  lore) {
                list.add(Component.text(color(string)));
            }
            itemMeta.lore(list);
        }
        if(glow) {
            itemMeta.addItemFlags(ItemFlag.HIDE_ENCHANTS);
            itemMeta.addEnchant(Enchantment.DURABILITY, 1, true);
        }
        if(unb) {
            itemMeta.setUnbreakable(true);
        }
        item.setItemMeta(itemMeta);
        return item;
    }

    public static ItemStack editItem(ItemStack item, String name,int amount, List<Component> lore) {
        if(amount == 0) {
            amount = 1;
        }
        item.setAmount(amount);
        ItemMeta itemMeta = item.getItemMeta();
        if (name != null) {
            itemMeta.displayName(Component.text(name));
        }
        itemMeta.lore(lore);
        item.setItemMeta(itemMeta);
        return item;
    }

    public static ItemStack closeItem () {
        return createItem(Material.BARRIER, 1, false, false, color("&cCLOSE"));
    }

    public static ItemStack greyPane() {
        return createItem(Material.GRAY_STAINED_GLASS_PANE, 1, false, true, "");
    }

    public static String itemStackSerilize(ItemStack it, Integer slot) {
        Gson gson = new Gson();
        //String str = gson.toJson(it.serialize());
        String str = gson.toJson(it.serialize(), new TypeToken<Map>(){}.getType());
        NBTItem nbti = new NBTItem(it);
        str = str + "@#NBT#" + nbti.toString();
        str = str + "@#SLOT#" + slot.toString();
        return str;
    }

    public static class itemStackDeserilizeResult {
        int slot;
        ItemStack it;

        public itemStackDeserilizeResult(Integer slot, ItemStack it) {
            this.slot = slot;
            this.it = it;
        }
    }

    public static itemStackDeserilizeResult itemStackDeserilize(String str) {
        List<String> mapp = Arrays.asList(str.split("@#NBT#"));
        Gson gson = new Gson();
        ItemStack it = new ItemStack(Material.AIR);
        Integer slot = 0;
        for (String s : mapp) {
            if(s != null && s.length() > 2) {
                try {
                    Map<String, Object> map = gson.fromJson(s, new TypeToken<Map<String, Object>>(){}.getType());
                    it = ItemStack.deserialize(map);
                }   catch (Exception e) {
                    List<String> mappp = Arrays.asList(s.split("@#SLOT#"));
                    for (String ss : mappp) {
                        try {
                            if (ss != null && ss.length() > 2) {
                                NBTItem nbti = new NBTItem(it);
                                String strrr = ss;
                                if (strrr.length() > 2) {
                                    for (String strr : strrr.split(",")) {
                                        if (!strr.equals("{") && !strr.equals("}")) {
                                            strr = strr.replace("{", "");
                                            strr = strr.replace("}", "");
                                            String[] nb = strr.split(":");
                                            nbti.setInteger(nb[0], Integer.valueOf(nb[1]));
                                            it = nbti.getItem();
                                        }
                                    }

                                }
                            }
                        } catch (Exception ee) {
                            slot = Integer.valueOf(ss);
                        }
                    }

                }

            }
        }
        itemStackDeserilizeResult itt  = new itemStackDeserilizeResult(slot, it);
        return itt;
    }

    public static String listItemStackSerelize(List<String> list) {
        Gson gson = new Gson();
        String str = "";
        for(String a : list) {
            str = str + "@#NEW_ITEM#" + a;
        }
        return str;
    }

    public static ItemStack[] listItemStackDeserilize(String str) {
        Gson gson = new Gson();
        List<String> map = Arrays.asList(str.split("@#NEW_ITEM#"));
        ArrayList<ItemStack> items = new ArrayList<ItemStack>();
        for(String s : map) {
            ItemStack itt;
            if(s != null) {
                itt = itemStackDeserilize(s).it;
                if (itt != null && itt.getType() != Material.AIR) {
                    items.add(itt);
                }
            }
        }
        ItemStack[] it = items.toArray(new ItemStack[0]);
        return it;
    }

    public static String attributes(String str) {
        String stt = "";
        if(str.contains("SPEED")) {
            stt = "&fSpeed";
            return stt;
        }
        if(str.contains("STRENGTH")) {
            stt = "&4Strength";
            return stt;
        }
        if(str.contains("HEALTH")) {
            stt = "&cHealth";
            return stt;
        }
        if(str.contains("DEFENSE")) {
            stt = "&1Defense";
            return stt;
        }
        else {
            stt = "None";
            return stt;
        }
    }

    public static  NBTItem setAttributes(String str, Integer nb, NBTItem nbti) {
        switch (str) {
            case "SPEED":
                nbti.setInteger("SPEED", nb);
            case "STRENGTH":
                nbti.setInteger("STRENGTH", nb);
            case "HEALTH":
                nbti.setInteger("HEALTH", nb);
            case "DEFENSE":
                nbti.setInteger("DEFENSE", nb);
            default:
                error("Cant set Attributes with an unknown attribute.");
        }
        return nbti;
    }

    public static Integer getHealth(ItemStack it) {
        NBTItem nbti = new NBTItem(it);
        if (nbti.hasKey("HEALTH")){
            return nbti.getInteger("HEALTH");
        }
        return 0;
    }
    public static Integer getSpeed(ItemStack it) {
        NBTItem nbti = new NBTItem(it);
        if (nbti.hasKey("SPEED")){
            return nbti.getInteger("SPEED");
        }
        return 0;
    }
    public static Integer getDefense(ItemStack it) {
        NBTItem nbti = new NBTItem(it);
        if (nbti.hasKey("DEFENSE")){
            return nbti.getInteger("DEFENSE");
        }
        return 0;
    }
    public static Integer getStrength(ItemStack it) {
        NBTItem nbti = new NBTItem(it);
        if (nbti.hasKey("STRENGTH")){
            return nbti.getInteger("STRENGTH");
        }
        return 0;
    }

    public static ItemStack setLoreFromNBT(ItemStack it) {
        ItemMeta itm = it.getItemMeta();
        List<Component> lore = new ArrayList<>();
        for (Component cp : itm.lore()) {
            lore.add(cp);
        }
        NBTItem nbti = new NBTItem(it);
        if(nbti.hasKey("SPEED")) {
            if(lore.size() > 2) {
                lore.add(2, Component.text(color(attributes("SPEED")) + nbti.getInteger("SPEED")));
            } else {
                lore.add(Component.text(color(attributes("SPEED")) + nbti.getInteger("SPEED")));
            }
        }
        if(nbti.hasKey("HEALTH")) {
            if(lore.size() > 1) {
                lore.add(1, Component.text(color(attributes("HEALTH")) + nbti.getInteger("HEALTH")));
            } else {
                lore.add(Component.text(color(attributes("HEALTH")) + nbti.getInteger("HEALTH")));
            }
        }
        if(nbti.hasKey("DEFENSE")) {
            if(lore.size() > 3) {
                lore.add(3, Component.text(color(attributes("DEFENSE")) + nbti.getInteger("DEFENSE")));
            } else {
                lore.add(Component.text(color(attributes("DEFENSE")) + nbti.getInteger("DEFENSE")));
            }
        }
        if(nbti.hasKey("STRENGTH")) {
            if(lore.size() > 3) {
                lore.add(3, Component.text(color(attributes("STRENGTH")) + nbti.getInteger("STRENGTH")));
            } else {
                lore.add(Component.text(color(attributes("STRENGTH")) + nbti.getInteger("STRENGTH")));
            }
        }
        itm.lore(lore);
        it.setItemMeta(itm);
        return it;
    }

    public static Attributes getNbtFromEntity(LivingEntity e) {
        EntityEquipment equi = e.getEquipment();
        Integer health = 0;
        Integer defense = 0;
        Integer speed = 0;
        Integer strength = 0;
        if (equi != null) {
            if (equi.getItemInMainHand().getType() != Material.AIR && equi.getHelmet() != null) {
                ItemStack item = equi.getItemInMainHand();
                health = health + getHealth(item);
                defense = defense + getDefense(item);
                speed = speed + getSpeed(item);
                strength = strength + getStrength(item);
            }
            if (equi.getHelmet().getType() != Material.AIR && equi.getHelmet() != null) {
                ItemStack item = equi.getHelmet();
                health = health + getHealth(item);
                defense = defense + getDefense(item);
                speed = speed + getSpeed(item);
                strength = strength + getStrength(item);
            }
            if (equi.getChestplate().getType() != Material.AIR && equi.getChestplate() != null) {
                ItemStack item = equi.getChestplate();
                health = health + getHealth(item);
                defense = defense + getDefense(item);
                speed = speed + getSpeed(item);
                strength = strength + getStrength(item);
            }
            if (equi.getChestplate().getType() != Material.AIR && equi.getChestplate() != null) {
                ItemStack item = equi.getChestplate();
                health = health + getHealth(item);
                defense = defense + getDefense(item);
                speed = speed + getSpeed(item);
                strength = strength + getStrength(item);
            }
            if (equi.getLeggings().getType() != Material.AIR && equi.getLeggings() != null) {
                ItemStack item = equi.getLeggings();
                health = health + getHealth(item);
                defense = defense + getDefense(item);
                speed = speed + getSpeed(item);
                strength = strength + getStrength(item);
            }
            if (equi.getBoots().getType() != Material.AIR && equi.getBoots() != null) {
                ItemStack item = equi.getBoots();
                health = health + getHealth(item);
                defense = defense + getDefense(item);
                speed = speed + getSpeed(item);
                strength = strength + getStrength(item);
            }
        }
        Attributes attributes = new Attributes(health, defense, speed, strength);
        return attributes;
    }

}
